@@ -12,6 +12,9 @@ pub struct Stream<D: Direction> {
     _marker: marker::PhantomData<D>,
 }
 
+unsafe impl<D: Direction> marker::Send for Stream<D> {}
+unsafe impl<D: Direction> marker::Sync for Stream<D> {}
+
 pub enum Compress {}
 pub enum Decompress {}
 
@@ -129,7 +132,7 @@ impl Stream<Decompress> {
                       output: &mut [u8],
                       flush: Flush)
                       -> c_int {
-        self.raw.next_in = input.as_ptr();
+        self.raw.next_in = input.as_ptr() as *mut u8;
         self.raw.avail_in = input.len() as c_uint;
         self.raw.next_out = output.as_mut_ptr();
         self.raw.avail_out = output.len() as c_uint;
