@@ -76,6 +76,7 @@ pub struct Config {
     compiler: Option<PathBuf>,
     archiver: Option<PathBuf>,
     cargo_metadata: bool,
+    pic: Option<bool>,
 }
 
 /// Configuration used to represent an invocation of a C compiler.
@@ -134,7 +135,8 @@ impl Config {
             env: Vec::new(),
             compiler: None,
             archiver: None,
-            cargo_metadata: true
+            cargo_metadata: true,
+            pic: None,
         }
     }
 
@@ -295,6 +297,15 @@ impl Config {
         self
     }
 
+    /// Configures whether the compiler will emit position independent code.
+    ///
+    /// This option defaults to `false` for `i686` and `windows-gnu` targets and to `true` for all
+    /// other targets.
+    pub fn pic(&mut self, pic: bool) -> &mut Config {
+        self.pic = Some(pic);
+        self
+    }
+
 
     #[doc(hidden)]
     pub fn __set_env<A, B>(&mut self, a: A, b: B) -> &mut Config
@@ -386,6 +397,9 @@ impl Config {
         self.print(&format!("debug={} opt-level={}", debug, opt_level));
 
         let mut cmd = self.get_base_compiler();
+        let nvcc = cmd.path.to_str()
+            .map(|path| path.contains("nvcc"))
+            .unwrap_or(false);
 
         if msvc {
             cmd.args.push("/nologo".into());
@@ -393,6 +407,8 @@ impl Config {
             if opt_level != 0 {
                 cmd.args.push("/O2".into());
             }
+        } else if nvcc {
+            cmd.args.push(format!("-O{}", opt_level).into());
         } else {
             cmd.args.push(format!("-O{}", opt_level).into());
             cmd.args.push("-ffunction-sections".into());
@@ -411,16 +427,23 @@ impl Config {
         } else if !msvc {
             if target.contains("i686") {
                 cmd.args.push("-m32".into());
-            } else if target.contains("x86_64") {
+            } else if target.contains("x86_64") || target.contains("powerpc64") {
                 cmd.args.push("-m64".into());
             }
 
-            if !target.contains("i686") && !target.contains("windows-gnu") {
+            if !nvcc && self.pic.unwrap_or(!target.contains("i686") && !target.contains("windows-gnu")) {
                 cmd.args.push("-fPIC".into());
+            } else if nvcc && self.pic.unwrap_or(false) {
+                cmd.args.push("-Xcompiler".into());
+                cmd.args.push("\'-fPIC\'".into());
             }
             if target.contains("musl") {
                 cmd.args.push("-static".into());
             }
+
+            if target == "armv7-unknown-linux-gnueabihf" {
+                cmd.args.push("-march=armv7-a".into());
+            }
         }
 
         if self.cpp && !msvc {
@@ -585,8 +608,11 @@ impl Config {
                 let prefix = match &target[..] {
                     "aarch64-unknown-linux-gnu" => Some("aarch64-linux-gnu"),
                     "arm-unknown-linux-gnueabi" => Some("arm-linux-gnueabi"),
-                    "arm-unknown-linux-gnueabihf" => Some("arm-linux-gnueabihf"),
+                    "arm-unknown-linux-gnueabihf"  => Some("arm-linux-gnueabihf"),
+                    "armv7-unknown-linux-gnueabihf" => Some("arm-linux-gnueabihf"),
                     "powerpc-unknown-linux-gnu" => Some("powerpc-linux-gnu"),
+                    "powerpc64-unknown-linux-gnu" => Some("powerpc-linux-gnu"),
+                    "powerpc64le-unknown-linux-gnu" => Some("powerpc64le-linux-gnu"),
                     "mips-unknown-linux-gnu" => Some("mips-linux-gnu"),
                     "i686-pc-windows-gnu" => Some("i686-w64-mingw32"),
                     "x86_64-pc-windows-gnu" => Some("x86_64-w64-mingw32"),
