@@ -67,7 +67,7 @@
 #![allow(trivial_numeric_casts, trivial_casts)]
 #![deny(missing_docs)]
 #![cfg_attr(test, deny(warnings))]
-#![cfg_attr(feature = "unstable", feature(catch_panic))]
+#![cfg_attr(feature = "unstable", feature(recover, std_panic))]
 
 extern crate libc;
 extern crate url;
@@ -90,6 +90,7 @@ pub use cred::{Cred, CredentialHelper};
 pub use describe::{Describe, DescribeFormatOptions, DescribeOptions};
 pub use diff::{Diff, DiffDelta, DiffFile, DiffOptions, Deltas};
 pub use diff::{DiffLine, DiffHunk, DiffStats, DiffFindOptions};
+pub use diff::{DiffBinary, DiffBinaryFile, DiffBinaryKind};
 pub use merge::{AnnotatedCommit, MergeOptions};
 pub use error::Error;
 pub use index::{Index, IndexEntry, IndexEntries, IndexMatchedPath};
@@ -241,7 +242,9 @@ pub enum RepositoryState {
     Clean,
     Merge,
     Revert,
+    RevertSequence,
     CherryPick,
+    CherryPickSequence,
     Bisect,
     Rebase,
     RebaseInteractive,
@@ -301,13 +304,15 @@ pub enum BranchType {
 /// searching for config entries.
 #[derive(PartialEq, Eq, Debug, Copy, Clone)]
 pub enum ConfigLevel {
+    /// System-wide on Windows, for compatibility with portable git
+    ProgramData,
     /// System-wide configuration file, e.g. /etc/gitconfig
     System,
     /// XDG-compatible configuration file, e.g. ~/.config/git/config
     XDG,
     /// User-specific configuration, e.g. ~/.gitconfig
     Global,
-    /// Reopsitory specific config, e.g. $PWD/.git/config
+    /// Repository specific config, e.g. $PWD/.git/config
     Local,
     /// Application specific configuration file
     App,
@@ -541,6 +546,7 @@ impl ConfigLevel {
     /// Converts a raw configuration level to a ConfigLevel
     pub fn from_raw(raw: raw::git_config_level_t) -> ConfigLevel {
         match raw {
+            raw::GIT_CONFIG_LEVEL_PROGRAMDATA => ConfigLevel::ProgramData,
             raw::GIT_CONFIG_LEVEL_SYSTEM => ConfigLevel::System,
             raw::GIT_CONFIG_LEVEL_XDG => ConfigLevel::XDG,
             raw::GIT_CONFIG_LEVEL_GLOBAL => ConfigLevel::Global,
