@@ -1,7 +1,9 @@
-extern crate pkg_config;
 extern crate cmake;
+extern crate gcc;
+extern crate pkg_config;
 
 use std::env;
+use std::ffi::OsString;
 use std::fs::{self, File};
 use std::io::prelude::*;
 use std::path::{Path, PathBuf};
@@ -32,6 +34,7 @@ fn main() {
     }
 
     let target = env::var("TARGET").unwrap();
+    let host = env::var("HOST").unwrap();
     let windows = target.contains("windows");
     let msvc = target.contains("msvc");
     let mut cfg = cmake::Config::new("libgit2");
@@ -64,6 +67,25 @@ fn main() {
         }
     }
 
+    // When cross-compiling, we're pretty unlikely to find a `dlltool` binary
+    // lying around, so try to find another if it exists
+    if windows && !host.contains("windows") {
+        let c_compiler = gcc::Config::new().cargo_metadata(false)
+                                           .get_compiler();
+        let exe = c_compiler.path();
+        let path = env::var_os("PATH").unwrap_or(OsString::new());
+        let exe = env::split_paths(&path)
+                      .map(|p| p.join(&exe))
+                      .find(|p| p.exists());
+        if let Some(exe) = exe {
+            if let Some(name) = exe.file_name().and_then(|e| e.to_str()) {
+                let name = name.replace("gcc", "dlltool");
+                let dlltool = exe.with_file_name(name);
+                cfg.define("DLLTOOL", &dlltool);
+            }
+        }
+    }
+
     if ssh {
         cfg.register_dep("SSH2");
     } else {
@@ -105,17 +127,20 @@ fn main() {
         return
     }
 
-    if env::var("HOST") == env::var("TARGET") {
-        // libssh2 is linked in elsehwere, don't want it reported via pkg-config
-        let pc = dst.join("lib/pkgconfig/libgit2.pc");
+    // libgit2 requires the http_parser library for the HTTP transport to be
+    // implemented, and it will attempt to use the system http_parser if it's
+    // available. Detect this situation and report using the system http parser
+    // the same way in this situation.
+    //
+    // Note that other dependencies of libgit2 like openssl, libz, and libssh2
+    // are tracked via crates instead of this. Ideally this should be a crate as
+    // well.
+    let pkgconfig_file = dst.join("lib/pkgconfig/libgit2.pc");
+    if let Ok(mut f) = File::open(&pkgconfig_file) {
         let mut contents = String::new();
-        t!(t!(File::open(&pc)).read_to_string(&mut contents));
-        let contents = contents.replace(" -lssh2 ", " ");
-        t!(t!(File::create(&pc)).write_all(contents.as_bytes()));
-
-        prepend("PKG_CONFIG_PATH", dst.join("lib/pkgconfig"));
-        if pkg_config::Config::new().statik(true).find("libgit2").is_ok() {
-            return
+        t!(f.read_to_string(&mut contents));
+        if contents.contains("-lhttp_parser") {
+            println!("cargo:rustc-link-lib=http_parser");
         }
     }
 
@@ -123,6 +148,8 @@ fn main() {
     println!("cargo:rustc-link-search=native={}", dst.join("lib").display());
     if target.contains("apple") {
         println!("cargo:rustc-link-lib=iconv");
+        println!("cargo:rustc-link-lib=framework=Security");
+        println!("cargo:rustc-link-lib=framework=CoreFoundation");
     }
 }
 
