@@ -1,6 +1,6 @@
 #![allow(non_camel_case_types, non_upper_case_globals, non_snake_case)]
 #![allow(dead_code)]
-#![doc(html_root_url="https://sfackler.github.io/rust-openssl/doc/v0.7.4")]
+#![doc(html_root_url="https://sfackler.github.io/rust-openssl/doc/v0.7.6")]
 
 extern crate libc;
 
@@ -22,9 +22,7 @@ pub type ENGINE = c_void;
 pub type EVP_CIPHER = c_void;
 pub type EVP_CIPHER_CTX = c_void;
 pub type EVP_MD = c_void;
-pub type EVP_PKEY = c_void;
 pub type EVP_PKEY_CTX = c_void;
-pub type RSA = c_void;
 pub type SSL = c_void;
 pub type SSL_CTX = c_void;
 pub type SSL_METHOD = c_void;
@@ -48,7 +46,6 @@ pub type bio_info_cb = Option<unsafe extern "C" fn(*mut BIO,
 
 #[repr(C)]
 #[derive(Copy, Clone)]
-#[allow(raw_pointer_derive)]
 pub struct BIO_METHOD {
     pub type_: c_int,
     pub name: *const c_char,
@@ -65,6 +62,47 @@ pub struct BIO_METHOD {
 // so we can create static BIO_METHODs
 unsafe impl Sync for BIO_METHOD {}
 
+#[repr(C)]
+pub struct RSA {
+    pub pad: c_int,
+    pub version: c_long,
+    pub meth: *const c_void,
+
+    pub engine: *mut c_void,
+    pub n: *mut BIGNUM,
+    pub e: *mut BIGNUM,
+    pub d: *mut BIGNUM,
+    pub p: *mut BIGNUM,
+    pub q: *mut BIGNUM,
+    pub dmp1: *mut BIGNUM,
+    pub dmq1: *mut BIGNUM,
+    pub iqmp: *mut BIGNUM,
+
+    pub ex_data: *mut c_void,
+    pub references: c_int,
+    pub flags: c_int,
+
+    pub _method_mod_n: *mut c_void,
+    pub _method_mod_p: *mut c_void,
+    pub _method_mod_q: *mut c_void,
+
+    pub bignum_data: *mut c_char,
+    pub blinding: *mut c_void,
+    pub mt_blinding: *mut c_void,
+}
+
+#[repr(C)]
+pub struct EVP_PKEY {
+    pub type_: c_int,
+    pub save_type: c_int,
+    pub references: c_int,
+    pub ameth: *const c_void,
+    pub engine: *mut ENGINE,
+    pub pkey: *mut c_void,
+    pub save_parameters: c_int,
+    pub attributes: *mut c_void,
+}
+
 #[repr(C)]
 pub struct BIO {
     pub method: *mut BIO_METHOD,
@@ -527,6 +565,9 @@ extern "C" {
     pub fn PEM_read_bio_PUBKEY(bio: *mut BIO, out: *mut *mut EVP_PKEY, callback: Option<PasswordCallback>,
                              user_data: *mut c_void) -> *mut X509;
 
+    pub fn PEM_read_bio_RSAPrivateKey(bio: *mut BIO, rsa: *mut *mut RSA, callback: Option<PasswordCallback>, user_data: *mut c_void) -> *mut RSA;
+    pub fn PEM_read_bio_RSA_PUBKEY(bio:    *mut BIO, rsa: *mut *mut RSA, callback: Option<PasswordCallback>, user_data: *mut c_void) -> *mut RSA;
+
     pub fn PEM_write_bio_PrivateKey(bio: *mut BIO, pkey: *mut EVP_PKEY, cipher: *const EVP_CIPHER,
                                     kstr: *mut c_char, klen: c_int,
                                     callback: Option<PasswordCallback>,
