@@ -396,7 +396,7 @@
 //! crate have time complexity `O(mn)` (with `m ~ regex` and `n ~ search
 //! text`), which means there's no way to cause exponential blow-up like with
 //! some other regular expression engines. (We pay for this by disallowing
-//! features like arbitrary look-ahead and back-references.)
+//! features like arbitrary look-ahead and backreferences.)
 
 #![deny(missing_docs)]
 #![cfg_attr(test, deny(warnings))]
@@ -420,6 +420,7 @@ mod backtrack;
 mod char;
 mod compile;
 mod input;
+mod inst;
 mod pool;
 mod prefix;
 mod program;
@@ -432,7 +433,8 @@ mod re;
 pub mod internal {
     pub use char::Char;
     pub use input::{Input, CharInput, InputAt};
-    pub use program::{Program, MatchEngine, CharRanges, Inst, LookInst};
+    pub use inst::{Inst, EmptyLook, InstRanges};
+    pub use program::{Program, MatchEngine};
     pub use re::ExNative;
     pub use re::Regex::{Dynamic, Native};
 }
